#lang racket

(require vigracket)
(require (rename-in 2htdp/image
                    (save-image save-plt-image)
                    (image-width plt-image-width)
                    (image-height plt-image-height)))
(require lang/posn)

;; HELPERS
;; -----------------------------------------------------------------------------
;; Bilder prozentual skalieren
(define (resizeimage-percent img p)
  (resizeimage img 
               (inexact->exact (round (* (image-width img)  (/ p 100))))
               (inexact->exact (round (* (image-height img) (/ p 100)))) 
               2))

;; BBox: 0 - left, 1 - upper, 2 - right, 3 - lower
(define (findBBox x y pixel bbox)
  (when (>  (car pixel) 0.0)
    (begin
      (when (> x (vector-ref bbox 2))    (vector-set! bbox 2 x))
      (when (< x (vector-ref bbox 0))    (vector-set! bbox 0 x))
      (when (> y (vector-ref bbox 3))    (vector-set! bbox 3 y))
      (when (< y (vector-ref bbox 1))    (vector-set! bbox 1 y)))))


;; Main program
;; -----------------------------------------------------------------------------
(define image_dir (current-directory))

(define img (load-image (build-path image_dir "IMG_8553.JPG")))

(define resized_img (resizeimage-percent img 10))

(define work_img (image->green resized_img))
(show-image work_img "Working Image")

;(show-image (cannyedgeimage work_img 1.0 0.0 255.0))
;(show-image (cannyedgeimage work_img 2.0 0.0 255.0))
;(show-image (cannyedgeimage work_img 3.0 0.0 255.0))

;(show-image (cannyedgeimage work_img 1.0 1.0 255.0))
;(show-image (cannyedgeimage work_img 1.0 5.0 255.0))
;(show-image (cannyedgeimage work_img 1.0 10.0 255.0))

(define canny_img (cannyedgeimage work_img 1.0 10.0 255.0))
(show-image canny_img "Canny Edge Detection")

;(show-image (image-map (lambda (x) (if (> x 100.0) 255.0 0.0)) work_img))
;(show-image (image-map (lambda (x) (if (> x 150.0) 255.0 0.0)) work_img))
;(show-image (image-map (lambda (x) (if (> x 200.0) 255.0 0.0)) work_img))

(define threshold_img (image-map (lambda (x) (if (> x 150.0) 255.0 0.0)) work_img))
(show-image threshold_img "Thresholding")

(define bbox1 (vector (image-width resized_img) (image-height resized_img) 0 0))
(define bbox2 (vector (image-width resized_img) (image-height resized_img) 0 0))

(image-for-each-pixel (curryr findBBox bbox1)  canny_img)
(image-for-each-pixel (curryr findBBox bbox2)  threshold_img)

(define (overlay-bboxes img bboxes colors)
  (if (empty? bboxes)
       (image->plt-image img)
       (let ((bbox  (car bboxes))
             (color (car colors)))
         (underlay/xy (overlay-bboxes img (cdr bboxes) (cdr colors))
                      (- (vector-ref bbox 0) 1)
                      (- (vector-ref bbox 1) 1)
                      (rectangle (+ (- (vector-ref bbox 2) (vector-ref bbox 0)) 2)
                                 (+ (- (vector-ref bbox 3) (vector-ref bbox 1)) 2)
                                 'outline color)))))

(define result1 (racket-image->image (overlay-bboxes resized_img (list bbox1) '(green))))
(define result2 (racket-image->image (overlay-bboxes resized_img (list bbox2) '(red))))

(define crop1_img (subimage resized_img (vector-ref bbox1 0)(vector-ref bbox1 1)(vector-ref bbox1 2)(vector-ref bbox1 3)))
(define crop2_img (subimage resized_img (vector-ref bbox2 0)(vector-ref bbox2 1)(vector-ref bbox2 2)(vector-ref bbox2 3)))

(show-image result1 "Result using Thresholding")
(show-image result2 "Result using Canny")